@@ -48,7 +48,9 @@ class PolicyFileTestCase(test.NoDBTestCase):
             # is_admin or not. As a side-effect, policy reset is needed here
             # to flush existing policy cache.
             policy.reset()
-            policy.init()
+            # NOTE(gmann): We do not need to log policy warnings for unit
+            # tests.
+            policy.init(suppress_deprecation_warnings=True)
             rule = oslo_policy.RuleDefault('example:test', "")
             policy._ENFORCER.register_defaults([rule])
 
@@ -86,7 +88,9 @@ class PolicyTestCase(test.NoDBTestCase):
                                     "role:ADMIN or role:sysadmin"),
         ]
         policy.reset()
-        policy.init()
+        # NOTE(gmann): We do not need to log policy warnings for unit
+        # tests.
+        policy.init(suppress_deprecation_warnings=True)
         # before a policy rule can be used, its default has to be registered.
         policy._ENFORCER.register_defaults(rules)
         self.context = context.RequestContext('fake', 'fake', roles=['member'])
@@ -232,7 +236,9 @@ class PolicyTestCase(test.NoDBTestCase):
 class IsAdminCheckTestCase(test.NoDBTestCase):
     def setUp(self):
         super(IsAdminCheckTestCase, self).setUp()
-        policy.init()
+        # NOTE(gmann): We do not need to log policy warnings for unit
+        # tests.
+        policy.init(suppress_deprecation_warnings=True)
 
     def test_init_true(self):
         check = policy.IsAdminCheck('is_admin', 'True')
@@ -317,7 +323,6 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:servers:show:host_status:unknown-only",
 "os_compute_api:servers:migrations:force_complete",
 "os_compute_api:servers:migrations:delete",
-"os_compute_api:os-admin-actions:reset_network",
 "os_compute_api:os-admin-actions:inject_network_info",
 "os_compute_api:os-admin-actions:reset_state",
 "os_compute_api:os-aggregates:index",
@@ -328,10 +333,6 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-aggregates:add_host",
 "os_compute_api:os-aggregates:remove_host",
 "os_compute_api:os-aggregates:set_metadata",
-"os_compute_api:os-agents:create",
-"os_compute_api:os-agents:update",
-"os_compute_api:os-agents:delete",
-"os_compute_api:os-baremetal-nodes",
 "os_compute_api:os-evacuate",
 "os_compute_api:os-extended-server-attributes",
 "os_compute_api:os-flavor-access:remove_tenant_access",
@@ -342,7 +343,10 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-flavor-manage:create",
 "os_compute_api:os-flavor-manage:update",
 "os_compute_api:os-flavor-manage:delete",
-"os_compute_api:os-hosts",
+"os_compute_api:os-hosts:update",
+"os_compute_api:os-hosts:reboot",
+"os_compute_api:os-hosts:shutdown",
+"os_compute_api:os-hosts:start",
 "os_compute_api:os-instance-actions:events",
 "os_compute_api:os-lock-server:unlock:unlock_override",
 "os_compute_api:os-migrate-server:migrate",
@@ -390,8 +394,6 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:server-metadata:update_all",
 "os_compute_api:os-suspend-server:suspend",
 "os_compute_api:os-suspend-server:resume",
-"os_compute_api:os-tenant-networks",
-"os_compute_api:extensions",
 "os_compute_api:servers:confirm_resize",
 "os_compute_api:servers:create",
 "os_compute_api:servers:create:attach_network",
@@ -419,13 +421,19 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-flavor-access",
 "os_compute_api:os-flavor-extra-specs:index",
 "os_compute_api:os-flavor-extra-specs:show",
-"os_compute_api:os-floating-ip-pools",
-"os_compute_api:os-floating-ips",
-"os_compute_api:os-multinic",
-"os_compute_api:os-networks:view",
+"os_compute_api:os-floating-ips:add",
+"os_compute_api:os-floating-ips:remove",
+"os_compute_api:os-floating-ips:create",
+"os_compute_api:os-floating-ips:delete",
+"os_compute_api:os-multinic:add",
+"os_compute_api:os-multinic:remove",
 "os_compute_api:os-rescue",
 "os_compute_api:os-unrescue",
-"os_compute_api:os-security-groups",
+"os_compute_api:os-security-groups:create",
+"os_compute_api:os-security-groups:update",
+"os_compute_api:os-security-groups:delete",
+"os_compute_api:os-security-groups:rule:create",
+"os_compute_api:os-security-groups:rule:delete",
 "os_compute_api:os-security-groups:add",
 "os_compute_api:os-security-groups:remove",
 "os_compute_api:os-server-password:clear",
@@ -439,7 +447,10 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-server-groups:delete",
 "os_compute_api:os-shelve:shelve",
 "os_compute_api:os-shelve:unshelve",
-"os_compute_api:os-volumes",
+"os_compute_api:os-volumes:create",
+"os_compute_api:os-volumes:delete",
+"os_compute_api:os-volumes:snapshots:create",
+"os_compute_api:os-volumes:snapshots:delete",
 "os_compute_api:os-volumes-attachments:create",
 "os_compute_api:os-volumes-attachments:delete",
 "os_compute_api:os-volumes-attachments:update",
@@ -449,9 +460,15 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-quota-sets:defaults",
 "os_compute_api:os-availability-zone:list",
 "os_compute_api:limits",
+"os_compute_api:extensions",
+"os_compute_api:os-floating-ip-pools",
 )
 
         self.system_reader_rules = (
+"os_compute_api:os-tenant-networks:list",
+"os_compute_api:os-tenant-networks:show",
+"os_compute_api:os-baremetal-nodes:list",
+"os_compute_api:os-baremetal-nodes:show",
 "os_compute_api:servers:migrations:index",
 "os_compute_api:servers:migrations:show",
 "os_compute_api:os-simple-tenant-usage:list",
@@ -460,7 +477,8 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-instance-actions:events:details",
 "os_compute_api:os-instance-usage-audit-log:list",
 "os_compute_api:os-instance-usage-audit-log:show",
-"os_compute_api:os-agents:list",
+"os_compute_api:os-hosts:list",
+"os_compute_api:os-hosts:show",
 "os_compute_api:os-hypervisors:list",
 "os_compute_api:os-hypervisors:list-detail",
 "os_compute_api:os-hypervisors:show",
@@ -469,10 +487,14 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-hypervisors:search",
 "os_compute_api:os-hypervisors:servers",
 "os_compute_api:limits:other_project",
+"os_compute_api:os-networks:list",
+"os_compute_api:os-networks:show",
 )
 
         self.system_reader_or_owner_rules = (
 "os_compute_api:os-simple-tenant-usage:show",
+"os_compute_api:os-security-groups:get",
+"os_compute_api:os-security-groups:show",
 "os_compute_api:os-security-groups:list",
 "os_compute_api:os-volumes-attachments:index",
 "os_compute_api:os-volumes-attachments:show",
@@ -483,6 +505,14 @@ class RealRolePolicyTestCase(test.NoDBTestCase):
 "os_compute_api:os-server-password:show",
 "os_compute_api:os-server-tags:index",
 "os_compute_api:os-server-tags:show",
+"os_compute_api:os-floating-ips:list",
+"os_compute_api:os-floating-ips:show",
+"os_compute_api:os-volumes:list",
+"os_compute_api:os-volumes:detail",
+"os_compute_api:os-volumes:show",
+"os_compute_api:os-volumes:snapshots:show",
+"os_compute_api:os-volumes:snapshots:list",
+"os_compute_api:os-volumes:snapshots:detail",
 )
 
         self.allow_nobody_rules = (
